package dynamictrees; 

/*
 * This code is based on an example provided by Richard Stanford, 
 * a tutorial reader.
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.Enumeration;

public class DynamicTree extends JPanel {
    protected DefaultMutableTreeNode rootNode;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();
    
    public JLabel text;
    public JPanel textPane;

    public DynamicTree() {
        rootNode = new DefaultMutableTreeNode("Root Node");
        treeModel = new DefaultTreeModel(rootNode);
        treeModel.addTreeModelListener(new MyTreeModelListener());

        tree = new JTree(treeModel);
        tree.setEditable(true);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);
        
        //Listen for when the tree expansionchanges.
        tree.addTreeExpansionListener(new TreeExpansionListener() {
        	
        	// Required by TreeExpansionListener interface.
        	public void treeExpanded(TreeExpansionEvent e) {
            	System.out.println("tree view was changed");
        		String tmp = "(tree view was changed) ";
        		tmp = tmp + " " + depthFirst();
        		text.setText(tmp);
        	}

        	// Required by TreeExpansionListener interface.
        	public void treeCollapsed(TreeExpansionEvent e) {
            	System.out.println("tree view was changed");
        		String tmp = "(tree view was changed) ";
        		tmp = tmp + " " + depthFirst();
        		text.setText(tmp);
        	}
        });
            
        JScrollPane scrollPane = new JScrollPane(tree);
        setLayout(new GridLayout(1,0));
        add(scrollPane);
        
        // TODO: Create the text viewing pane -- 
        // this could be done more elaborately -- be creative!	
     	text = new JLabel();
		textPane = new JPanel();
		textPane.add(text);
		textPane.setBackground(Color.white);
    }
    

    /** Remove all nodes except the root node. */
    public void clear() {
        rootNode.removeAllChildren();
        treeModel.reload();
    }

    /** Remove the currently selected node. */
    public void removeCurrentNode() {
        TreePath currentSelection = tree.getSelectionPath();
        if (currentSelection != null) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)
                         (currentSelection.getLastPathComponent());
            MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
            if (parent != null) {
                treeModel.removeNodeFromParent(currentNode);
                return;
            }
        } 

        // Either there was no selection, or the root was selected.
        toolkit.beep();
    }

    /** Add child to the currently selected node. */
    public DefaultMutableTreeNode addObject(Object child) {
        DefaultMutableTreeNode parentNode = null;
        TreePath parentPath = tree.getSelectionPath();

        if (parentPath == null) {
            parentNode = rootNode;
        } else {
            parentNode = (DefaultMutableTreeNode)
                         (parentPath.getLastPathComponent());
        }

        return addObject(parentNode, child, true);
    }

    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent,
                                            Object child) {
        return addObject(parent, child, false);
    }

    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent,
                                            Object child, 
                                            boolean shouldBeVisible) {
        DefaultMutableTreeNode childNode = 
                new DefaultMutableTreeNode(child);

        if (parent == null) {
            parent = rootNode;
        }

        treeModel.insertNodeInto(childNode, parent, 
                                 parent.getChildCount());

        // Make sure the user can see the lovely new node.
        if (shouldBeVisible) {
            tree.scrollPathToVisible(new TreePath(childNode.getPath()));
        }
        return childNode;
    }
    
       
    // TODO:  this method just uses a built in iterator
    // to return a string -- 
    // we suggest you also try to write your own... to gain experience.
    // Also -- the output here is really ugly, try to improve it!
    // Also try to do both preorder and postorder traversals of the tree
    // the traversals should only work for visible nodes....
    public String depthFirst() {
    	
    	Enumeration myit = rootNode.preorderEnumeration();
    	
    	String tmp = "Preorder Traversal: ";
    	while (myit.hasMoreElements()) {
    		DefaultMutableTreeNode next = (DefaultMutableTreeNode)myit.nextElement();
    		if ( tree.isVisible( new TreePath(next.getPath()))  ) {
    			tmp = tmp + " " + next.toString();
    		}
    	}	
    	return tmp;
    }
    	

    class MyTreeModelListener implements TreeModelListener {
        public void treeNodesChanged(TreeModelEvent e) {
            DefaultMutableTreeNode node;
            node = (DefaultMutableTreeNode)
                     (e.getTreePath().getLastPathComponent());

            /*
             * If the event lists children, then the changed
             * node is the child of the node we've already
             * gotten.  Otherwise, the changed node and the
             * specified node are the same.
             */
            try {
                int index = e.getChildIndices()[0];
                node = (DefaultMutableTreeNode)
                       (node.getChildAt(index));
            } catch (NullPointerException exc) {}

            System.out.println("The user has finished editing the node.");
            System.out.println("New value: " + node.getUserObject());
                    	System.out.println("tree was cleared");
        	String tmp = "Finished renaming a node";
        	tmp = tmp + " " + depthFirst();
        	text.setText(tmp);
        }
        public void treeNodesInserted(TreeModelEvent e) {
        	System.out.println("node inserted");
        	String tmp = "(node inserted) ";
        	tmp = tmp + " " + depthFirst();
        	text.setText(tmp);
        }
        public void treeNodesRemoved(TreeModelEvent e) {
        	System.out.println("node removed");
        	String tmp = "(node REMOVED) ";
        	tmp = tmp + " " + depthFirst();
        	text.setText(tmp);
        	// end assign4
        }
        public void treeStructureChanged(TreeModelEvent e) {
        	System.out.println("tree was cleared");
        	String tmp = "(tree was cleared) ";
        	tmp = tmp + " " + depthFirst();
        	text.setText(tmp);
        }
      
    }
   
}

