//Music.java, adapted from Bruce Eckel's Book, Thinking in Java
package music; 

interface Instrument {
  // Compile-time constant:
  int i = 5; // static & final by default
  // Cannot have method definitions:
  void play(); // all three Automatically public
  String what();
  void adjust();
}

class Wind implements Instrument {
  public void play() {
    System.out.println("Wind.play()");
  }
  public String what() { return "Wind"; }
  public void adjust() {}  // empty implementation
}

class Percussion implements Instrument {
  public void play() {
    System.out.println("Percussion.play()");
  }
  public String what() { return "Percussion"; }
  public void adjust() {}
}

class Stringed implements Instrument {
  public void play() {
    System.out.println("Stringed.play()");
  }
  public String what() { return "Stringed"; }
  public void adjust() {}
}

class Brass extends Wind {
  public void play() {
    System.out.println("Brass.play()");
  }
  public void adjust() { 
    System.out.println("Brass.adjust()");
  }
}

class Woodwind extends Wind {
  public void play() {
    System.out.println("Woodwind.play()");
  }
  public String what() { return "Woodwind"; }
  // EXERCISE: do we need an adjust implementation, why not? 
}

public class Music {
  // This class doesn't care about type of instrument, so new types
  // added to the system still work right:
  static void tune(Instrument i) {
    // ...
	i.adjust(); 
    i.play();
  }
  static void tuneAll(Instrument[] e) {
    for(int i = 0; i < e.length; i++)
      tune(e[i]);
  }
  public static void main(String[] args) {
    Instrument[] orchestra = new Instrument[5];
    int i = 0;
    // Upcasting during addition to the array:
    orchestra[i++] = new Wind();
    orchestra[i++] = new Percussion();
    orchestra[i++] = new Stringed();
    orchestra[i++] = new Brass();
    orchestra[i++] = new Woodwind();
    tuneAll(orchestra);
    /* expected output:  
      "Wind.play()",
      "Percussion.play()",
      "Stringed.play()",
      "Brass.adjust()"
      "Brass.play()",
      "Woodwind.play()"
    */
  }
} 
