/* This example looks at exceptions using the Snow class, which can throw two types
 * of exception.  
 *    
 * The key ideas to come away with are:
 * - When unexpected situations occurs an exception can be thrown. 
 * - An exception is an object which can be caught and handled.
 * - Try-catch blocks handle exceptions
 * - We can use predefined exception types (e.g. NullPointerException, ArrayIndexOutOfBoundsException)
 * 		(or we could create our own types - not covered here).  
 * - Information about objects can be found by calling the object's toString() method.
 * - this example also demonstrates reading input from the console 
 */

import java.io.*;

public class Snow{
	
	private static BufferedReader stdin;
		
	/* Since we are using IO there are many potential errors that could occur,
	 * so this method can throw an "IOException".  Whenever a method could potentially 
	 * throw an exception we should include that in the method signature.
	 * In this case, we are simply assigning the input to a String.
	 * Not much can go wrong here.
	 */	
	public static void boardOrSkis() throws IOException{
		System.out.print("Snowboard or skis? ");
		String sport = stdin.readLine();  // note -- if we enter an integer -- is it ok, why?
	}
	
	
	public static void getDay() throws IOException{
		System.out.print("What day is it today? ");
		String day = stdin.readLine();
	}
	
	/* Now, here there's a possible problem!  
	 * In this next example, we are expecting an integer!
	 * What happens if the input isn't an integer?
	 * Try it and see.
	 *
	 * Now try to fix it... 
	 * EXERCISE 1: Remove the comment out marks from
	 *  the try-catch block and run to see what happens.
	 * (Make sure you uncomment both parts (lines 48, 50-59 but leave out 
	 * 55 and 56 for now) ).
	 * See how we catch the NumberFormatException that was created?
	 */
	public static void numberOfClasses() throws IOException, NumberFormatException{
		System.out.print("How many classes do you have? ");
//		try{
			int numClasses = Integer.parseInt( stdin.readLine() );
//			System.out.println("****** printlin after exception ****");	
//		}catch(NumberFormatException e){
//			System.out.println("Was that a number?  I don't think so....");	
//			// debug(e);  // EXERCISE 2: now remove these comments! 
//			// numberOfClasses(); // try again!
//			return; 
//		} finally {
//			System.out.println("**this should always be printed in finally**");
//		}
	}
	
	
	/* Exceptions are objects!  And objects can store valuable data.
	 * We can print out that data by calling the object's "toString()" method.
 	 * This can be quite useful when you are unsure where a problem is occuring.
	 * Uncomment line 51 above to test.
	 */
	public static void debug(Exception e){
		String debugInfo = e.toString();
		System.out.println(debugInfo);
		
		// We could also do the following -- 
		//Remember Java calls the object's toString() method for us!
		// System.out.println(e);
	}

    public static void main(String [] args) throws IOException{

		stdin = new BufferedReader(new InputStreamReader(System.in));
		  
		System.out.println("It's still winter, but where's all the snow?");
		System.out.println();
		System.out.println("This program uses basic input and output to demo some aspects of exceptions.");
		System.out.println("When a question arises, type your answer into after the question in " +
			"the console and press return.");
		System.out.println();
		
		boardOrSkis();
		getDay();
		numberOfClasses();
		System.out.println("Goodbye");
	}	
}