// Example of creating, throwing and handling exceptions
// Exception classes usually:
// - have constructors
// - have no methods or data

// Below are three exception classes that we define
// These classes don't do very much
// They only call their superclass constructor

class Exception1 extends Exception  {
	public Exception1() {   
		super();   // recall don't need this explicit call....
	}
	public Exception1(String s) {   // a constructor that takes a string
		super(s);
	}
}

class Exception2 extends Exception  {
	public Exception2() {   // is this needed?
		super();  // don't really need this call.....
	}
	public Exception2(String s) {
		super(s); // but we do need this one
	}
}

class ExceptionSub1 extends Exception1  {
	public ExceptionSub1() { 
		super(); 
	}
	public ExceptionSub1(String s) {
		super(s);
	}
}

public class TestExceptions {
	
	// main()catches two standard Java exceptions
	
	public static void main (String [] args) {
		int myInt;
		
		// Convert a command line argument to an integer
		// Exceptions occur if an argument is not available 
		// or if it cannot be converted to an integer
		try {
			myInt = Integer.parseInt(args[0]);
		} catch (ArrayIndexOutOfBoundsException e) {  // if args is empty
			System.out.println("No command line argument provided.");
			return;
		} catch (NumberFormatException e) {  // cannot be converted to int
			System.out.println("You must provide an integer.");
			return;
		} finally {
			System.out.println("finally do something");	
		}
		System.out.println("outside try/catch/finally doing something when!"); 
		// Do something interesting!!!
		first(myInt);
	} // main
			 
	
	// Method first() 
	// calls second() which throws Exception1.
	// Exception1 is handled here.
	
	public static void first(int x) {
		try {
			second(x);
		} catch (Exception1 e) {   // Handle Exception1 and ExceptionSub1
			if (e instanceof ExceptionSub1) {
				System.out.println("ExceptionSub1: ");
			} else {
				System.out.println("Exception1: ");
			}
			System.out.println(e.getMessage()); // gets string error message from e
			System.out.println("Handled in first.");
		}  // catch
	} // first
	
	// Method second()
	// calls third() which throws Exception1 and Exception2
	// Handles Exception2 only
	// Exception1 must therefore be declared in this methods throw clause
	
	public static void second(int x)throws Exception1 {
		int r;
		try {
			System.out.println("x is " + x + "\n");
			r = third(x);
			System.out.println("third(x) is " + r);
		} catch (Exception2 e) {	// Handle Exception2
			System.out.println("Exception2: " + e.getMessage());
			System.out.println("Handled in second");
		} finally {
			// This code is executed after a local catch but before 
			// any other catch clauses
			System.out.println("\nfinally in second(): Note where other catches are.\n");
		}
	}
	
	// Method third()
	// Squares a parameter or throws an exception
	// Although it throws Exception1, Exception2 and ExceptionSub1
	// only the first two must be listed in the throw clause.  This
	// is because ExceptionSub1 is a subclass of Exception1.
	
	public static int third(int x) throws Exception1, Exception2
	{
		if (x < 0)
			throw new Exception1(x + " is too low");
		else if (x == 0) 
			throw new ExceptionSub1(x + " is still too low!");
		else if (x >= 100)
			throw new Exception2(x + " is too high");
		else
			return x*x;
	}
			
}