import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class SimpleFractal {

	private final static int INITIAL_WIDTH = 500; 
	private final static int INITIAL_HEIGHT = 300; 
	private final static double REDUCTION = 1.2;
	
	private static Color [] colors = {Color.black, Color.white};
	private static int next = 0;
	
	public static void draw(Graphics g, int x, int y, int width, int height) {
		if (height < 1)
			return;

		g.fillOval(x, y, width, height);
		
		Color color = getNextColor();
		g.setColor(color);
		int newWidth = (int) (width/REDUCTION);
		int newHeight = (int) (height/REDUCTION);
		int newX = x;
		int newY = (int) (y + (height - newHeight)/2.0);
		draw(g, newX, newY, newWidth, newHeight);
	}
	
	public static Color getNextColor () {
		Color nextColor = colors[next];
		if (next == colors.length-1) {
			next = 0;
		} else {
			next++;
		}
		return nextColor;
	}
	
	public static void main(String[] args) {	
		JFrame frame = new JFrame ("Simple Fractal");
		JPanel canvas = new JPanel () {
			public void paint(Graphics g) {
				((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				g.setColor (Color.white); 
				g.fillRect (0, 0, this.getWidth(), this.getHeight());
				
				g.setColor (getNextColor()); 
				draw (g, 0, 0, this.getWidth(), this.getHeight()); 
			}
		};
		canvas.setBackground(Color.white);
		canvas.setPreferredSize(new Dimension (INITIAL_WIDTH, INITIAL_HEIGHT));
		frame.getContentPane().add(canvas);
		frame.pack();
		frame.show();
	}

}
