package doublelinkedlist;

public class DoubleLinkedList {

	private DNode head;
	private DNode tail;
	private int size;

	public DoubleLinkedList() {
		head = null;
		tail = null;
		size = 0;
	}
	public DNode getHead() {
		return head;
	}
	public DNode getTail() {
		return tail;
	}
	public boolean isEmpty() {
		return head == null;
	}
	public int size() {
		return size;
	}
	public Object getFirst() {
		if (head == null) {
			return null;
		} else {
			return head.getData();
		}
	}
	public Object getLast() {
		if (tail == null) {
			return null;
		} else {
			return tail.getData();
		}
	}
	public void insertFirst(Object data) {
		DNode node = new DNode(data);
		size++;
		if (head == null) {
			head = node;
			tail = node;
		} else {
			node.setNext(head);
			head.setPrev(node);
			head = node;
		}
	}
	public void insertLast(Object data) {
		DNode node = new DNode(data);
		size++;
		if (tail == null) {
			head = node;
			tail = node;
		} else {
			tail.setNext(node);  // make tail's next point to node
			node.setPrev(tail);  // make new one's prev point to tail
			tail = node;		// make this new one the tail
		}
	}
	public Object deleteFirst() {
		if (head == null) {
			return null;
		} else {
			DNode aux = head;
			head = aux.getNext();
			aux.setNext(null);
			if (head == null) {
				tail = null;
			} else {
				head.setPrev(null);
			}
			size--;
			return aux.getData();
		}
	}
	public Object deleteLast() {
		if (tail == null) {
			return null;
		} else {
			DNode aux = tail;
			tail = aux.getPrev();
			aux.setPrev(null);
			if (tail == null) {
				head = null;
			} else {
				tail.setNext(null);
			}
			size--;
			return aux.getData();
		}
	}
	public String toString() {
		String str = new String();
		DNode n = head;
		if (head == null)
			return "Empty";
		while (n != null) {
			str = str + "[" + n.getData().toString() + "]";
			n = n.getNext();
		}
		return str;
	}
	public String toStringInReverse() {
		String str = new String();
		DNode n = tail;
		if (tail == null)
			return "Empty";
		while (n != null) {
			str = str + "[" + n.getData().toString() + "]";
			n = n.getPrev();
		}
		return str;
	}
	
	public static void main(String[] args) {
		DoubleLinkedList dl = new DoubleLinkedList();
		Integer i, j, k;
		
		i = new Integer(21);
		j = new Integer(15);
		k = new Integer(33);
		dl.insertFirst(i); // add 21  [21]
		dl.insertFirst(j);  // insert 15 before 21  [15, 21]
		dl.insertLast(k); // insert 33 after 21 [15, 21, 33]
		
		System.out.println("Length is " + dl.size());
		System.out.println("Elements are " + dl.toString());   // prints the above out
		System.out.println("Length is " + dl.size());
		System.out.println("Elements are " + dl.toStringInReverse());   // prints the above out in reverse
		
		dl.deleteFirst();
		System.out.println("Length is " + dl.size());
		System.out.println("Elements are " + dl.toString());   // prints the above out
		System.out.println("Elements are " + dl.toStringInReverse());   // prints the above out in reverse
		
		dl.deleteLast();
		System.out.println("Length is " + dl.size());
		System.out.println("Elements are " + dl.toString());   // prints the above out
		System.out.println("Elements are " + dl.toStringInReverse());   // prints the above out in reverse

	}
}
