/**
 * Evaluator.java
 * @version 1.0.1
 * @author Bette Bultena
 */

package evaluator; 

import java.util.StringTokenizer;
import java.io.*;

/**
 * Evaluator class evaluates legitimate postfix expressions using numbers
 * from StandardIn and sends the result to StandardOut.
 * Currently this version does not handle negative integers in the
 * expression.
 */
public class Evaluator {

	private static final char DIVIDE = '/';
	private static final char TIMES = '*';
	private static final char PLUS = '+';
	private static final char MINUS = '-';

/**
 * Takes a valid postfix expression involving only integers,
 * and +, -, *, / operators and returns an integer result.
 * @param input A valid postfix string.
 * @return The integer result.
 * @throws RuntimeException if the expression is not valid.
 */
	public static double postfix( String input ) {
		String token, operand1, operand2, finalResult;
		SimpleStack stack;
		StringTokenizer tk;
		double temp, op1, op2;
		double result = 0;

		tk = new StringTokenizer( input );
		stack = new SimpleStack();
		while (tk.hasMoreTokens()) {
			token = tk.nextToken();
			try {
				temp = Double.parseDouble( token );
				stack.push( token );
			}
			catch (NumberFormatException e) { // it is probably an operator
				try {
					operand2 = (String)stack.pop();
					operand1 = (String)stack.pop();
					op1 = Double.parseDouble(operand1);
					op2 = Double.parseDouble(operand2);
				}
				catch (RuntimeException exc) { // the stack is empty
					throw new RuntimeException( "Too many operators" );
				}

				switch (token.charAt(0)) {
					case DIVIDE:
						result = op1 / op2;
						if (result == Double.POSITIVE_INFINITY || result == Double.NEGATIVE_INFINITY) {
							throw new RuntimeException( "Divide by zero" );
						}
						break;
					case TIMES:
						result = op1 * op2;
					    break;
					case PLUS:
						result = op1 + op2;
						break;
					case MINUS:
						result = op1 - op2;
						break;
					default:
						throw new RuntimeException( "Illegal operator" );
				}
				stack.push( "" + result );
            } // catch NumberFormatException
		} // while loop
		finalResult = (String)stack.pop();
		if (!stack.isEmpty()) {
			throw new RuntimeException( "Too many operands" );
		}
		return Double.parseDouble( finalResult );
	}
	public static void main( String[] args ) {
		BufferedReader stdin;
		String userInput;
		double result;

		stdin = new BufferedReader( new InputStreamReader( System.in ));
		while (true) {
			System.out.print("Input a string, <enter> to stop: ");
			System.out.flush();
			try {
				if ((!(userInput = stdin.readLine()).equals(""))) {
					System.out.println("Result = "+Evaluator.postfix( userInput ));
				} else {
					break;
				}
			} catch (Exception e) {
				System.out.println( e.getMessage() + " try again" );
				continue;
			}
		}
	}
}